/* GFX.cpp
	Designed as a wrapper for graphics libraries. Using defines, we should be able to alter
	which graphics library we use to increase portability. Right now only allegro is supported.
	It is also a singleton, and is used as the single entry point for drawing to the screen.
*/

#include <Math.h>
#include "Game.h"
#ifdef BUILD_ALLEGRO
#include <allegro.h>
#endif

GFX* GFX::instance = 0;

GFX* GFX::Instance() {
	if(instance == 0) {
		instance = new GFX();
	}
	return instance;
}

GFX::GFX() {
	init();
}

/*
// Returns the instance if it has just been initialized with the given values
// null if it already exists
GFX* GFX::InitInstance(GFXmode gfxmode, int screenh, int screenv) {
	if(instance == 0) {
		instance = new GFX(gfxmode, screenh, screenv);
		return instance;
	}
	return 0;
}

GFX::GFX(GFXmode gfxmode, int screenh, int screenv) {
	init(gfxmode, screenh, screenv);
}*/

// Below are our wrapped allegro calls.
// We'd replace these with openGL calls or whatever when adding support.
#ifdef BUILD_ALLEGRO
GFX::~GFX() {
	destroy_bitmap(GFXbuffer);
}

void GFX::GFXrectfill(int x1, int y1, int x2, int y2, Color col) {
	rectfill(GFXbuffer, x1, y1, x2, y2, makecol(col.r, col.g, col.b));
}

void GFX::GFXtriangle(int x1, int y1, int x2, int y2, int x3, int y3, Color col) {
	triangle(GFXbuffer, x1, y1, x2, y2, x3, y3, makecol(col.r, col.g, col.b));
}

void GFX::GFXline(int x1, int y1, int x2, int y2, Color col) {
	fastline(GFXbuffer, x1, y1, x2, y2, makecol(col.r, col.g, col.b));
}

void GFX::GFXclearTo(Color col) {
	clear_to_color(GFXbuffer, makecol(col.r, col.g, col.b));
}

void GFX::GFXputpixel(int x, int y, Color col) {
	putpixel(GFXbuffer, x, y, makecol(col.r, col.g, col.b));
}

/*
Vec2 GFX::GetScreenSize() {
	return Vec2(SCREENH, SCREENV);
}*/

void GFX::init() {
	set_color_depth(16);
	set_gfx_mode(GFX_AUTODETECT, SC_WIDTH, SC_HEIGHT, 0, 0);
	GFXbuffer = create_bitmap(SCREEN_W, SCREEN_H);
}

/*
void GFX::init(GFXmode gfxmode, int screenh, int screenv) {
	SCREENH = screenh;
	SCREENV = screenv;
	set_color_depth(16);
	set_gfx_mode(gfxmode, SCREENH, SCREENV, 0, 0);
	GFXbuffer = create_bitmap(SCREENH, SCREENV);
}*/

void GFX::GFXupdate() {
	blit(GFXbuffer, screen, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
}

void GFX::GFXtext(char *str, int x, int y, Color col, Color mask) {
	textout_ex(GFXbuffer, font, str, x, y,makecol(col.r, col.g, col.b), makecol(mask.r, mask.g, mask.b));
}

void GFX::GFXtext_center(char *str, int x, int y, Color col, Color mask) {
	textout_centre_ex(GFXbuffer, font, str, x, y, makecol(col.r, col.g, col.b), makecol(mask.r, mask.g, mask.b));
}
#endif