#pragma once
#include <allegro.h>
#include <string>
#include "Enums.h"
#include "Vec2.h"

using namespace std;

// A generic rgb color struct. Used for storing / adding colors.
struct Color {
	int r;
	int g;
	int b;

	Color() {
		r = 0;
		g = 0;
		b = 0;
	}

	Color(int nr, int ng, int nb) {
		r = nr;
		g = ng;
		b = nb;
	};

	Color operator+( Color &other ){
		r += other.r;
		g += other.g;
		b += other.b;
		if(r < 0)	r = 0;
		if(r > 255) r = 255;
		if(g < 0)	g = 0;
		if(g > 255) g = 255;
		if(b < 0)	b = 0;
		if(b > 255) b = 255;

		return Color(r,g,b);
	};
	Color operator-( Color &other ){
		return Color(r,g,b) + Color(-other.r, -other.g, -other.b);
	};	

};

// Our graphics wrapper. All draw calls go through this class.
class GFX {
public:

	static GFX* Instance();
	void GFXtriangle(int x1, int y1, int x2, int y2, int x3, int y3, Color col);
	void GFXrectfill(int x1, int y1, int x2, int y2, Color col);
	void GFXline(int x1, int y1, int x2, int y2, Color col);
	void GFXputpixel(int x, int y, Color col);
	void GFXclearTo(Color col);
	void GFXupdate();
	void GFXtext(char *str, int x, int y, Color col, Color mask);
	void GFXtext_center(char *str, int x, int y, Color col, Color mask);
	BITMAP *GFXbuffer;
	~GFX();
	void init();
protected:
	GFX();
	GFX(GFXmode gfxmode, int screenh, int screenv);
private:
	// Will be used for resolution stuff in the future
	int SCREENH;
	int SCREENV;
	static GFX* instance;
};
